#include "BustaEngine.h"

BustaEngine::BustaEngine(){
	pause = 1000;		//time between first keydown and repeating, in ms
	interval = 250;		//interval between repeating key events (ms)	
	multiplayer = false;
	srand((unsigned)time(NULL));

		// init SDL
	if (SDL_Init(0) == -1)
	{
		printf("SDL_Init: %s\n", SDL_GetError());
	}

	int enableRepeat = SDL_EnableKeyRepeat(pause,interval);

	if(enableRepeat==-1)
		printf("Error enabling key repeat");

	network = new BustaNetClient();

	graphics = new BustaGraphics();
	gs = new BustaModelGameState();

	state = MAIN_MENU;

}

BustaEngine::~BustaEngine(){
	delete network;
	delete graphics;
	delete gs;

}

ScreenState BustaEngine::getState(){
	return state;
}

void BustaEngine::setState(ScreenState s){
	state = s;
}

void BustaEngine::mainMenu(){
	const int I_PICK = SDLK_RETURN;
	const int I_UP = SDLK_UP;
	const int I_DOWN = SDLK_DOWN;

	printf("main menu");

	MainScrnButton button = MS_START_SP;
	
	graphics->drawMainScreen(button);

	bool done = false;

	while(!done)
	{
		SDL_Event event;

		while ( SDL_PollEvent(&event) && !done)
		{
			graphics->drawMainScreen(button);

			switch(event.type){
				case SDL_KEYDOWN:
					printf("We got a keyboard event %d.\n", event.key.keysym.sym);
					if(event.key.keysym.sym==I_PICK){
						if(button==MS_START_SP){
							printf("Going to character select(SP) screen\n");
							multiplayer = false;
							state = CHAR_SELECT;
							done = true;
						}
						else if(button==MS_START_MP){
							printf("Going to connect screen\n");
							multiplayer = true;
							state = CONNECT;
							done = true;
						}
						else if (button==MS_HIGH_SCORE){
							printf("Going to high scores screen\n");
							state = HIGH_SCORES;
							done = true;
						}
						else if (button==MS_OPTIONS){
							printf("Going to high scores screen\n");
							state = OPTIONS;
							done = true;
						}
						else if(button==MS_QUIT){
							printf("Exiting loop\n");
							state = EXIT;
							done = true;
						}
					}
					else if(event.key.keysym.sym==I_UP){
						printf("moving up");
						button= (MainScrnButton)((button-1+MS_NUM_BUTTONS)%MS_NUM_BUTTONS);
					}
					else if(event.key.keysym.sym==I_DOWN){
						button= (MainScrnButton)((button+1)%MS_NUM_BUTTONS);
					}
					break;
				case SDL_MOUSEMOTION:
					for(int i =0; i<MS_NUM_BUTTONS; i++){
						if(buttonSelected(graphics->bMainScrn[i],event.motion.x,event.motion.y)){
							button = (MainScrnButton)i;
							break;
						}
					}
					break;
				case SDL_MOUSEBUTTONUP:
					if(event.button.button==SDL_BUTTON_LEFT&&buttonSelected(graphics->bMainScrn[button],event.button.x,event.button.y)){
						if(button==MS_START_SP){
							printf("Going to character select(SP) screen\n");
							multiplayer = false;
							state = CHAR_SELECT;
							done = true;
						}
						else if(button==MS_START_MP){
							printf("Going to connect screen\n");
							multiplayer = true;
							state = CONNECT;
							done = true;
						}
						else if (button==MS_HIGH_SCORE){
							printf("Going to high scores screen\n");
							state = HIGH_SCORES;
							done = true;
						}
						else if (button==MS_OPTIONS){
							printf("Going to high scores screen\n");
							state = OPTIONS;
							done = true;
						}
						else if(button==MS_QUIT){
							printf("Exiting loop\n");
							state = EXIT;
							done = true;
						}
					}
						
				break;
				default:
					printf("Unknown input, try again!\n");
					break;
			}
		}
	} 
}

void BustaEngine::optionsMenu(){
	const int I_KEY_BINDINGS = SDLK_k;
	const int I_SOUNDS = SDLK_s;
	const int I_DIFFICULTY = SDLK_d;
	const int I_MAIN_MENU = SDLK_m;
	
	printf("entered options menu\n");
	graphics->drawOptions();

	bool done = false;

	while(!done)
	{
		SDL_Event event;

		while ( SDL_PollEvent(&event) && !done)
		{
			switch(event.type){
				case SDL_KEYDOWN:
					printf("We got a keyboard event.\n");
					if(event.key.keysym.sym==I_KEY_BINDINGS){
						printf("Going to key binding screen\n");
						state = KEY_BINDINGS;
						done = true;
					}
					else if(event.key.keysym.sym==I_SOUNDS){
						printf("Going to sounds menu\n");
						state = SOUNDS;
						done = true;
					}
					else if(event.key.keysym.sym==I_DIFFICULTY){
						printf("Going to difficulty menu\n");
						state = DIFFICULTY;
						done = true;
					}
					else if(event.key.keysym.sym==I_MAIN_MENU){
						printf("Going back to Main Menu\n");
						state = MAIN_MENU;
						done = true;
					}
					break;
				default:
					printf("Unknown input, try again!\n");
					break;
			}
		}
	} 
}

void BustaEngine::highScores(){
	const int I_OK = SDLK_o;

	graphics->drawHighScores();
	printf("entered high scores menu\n");

	bool done = false;

	while(!done)
	{
		SDL_Event event;

		while ( SDL_PollEvent(&event) && !done)
		{
			switch(event.type){
				case SDL_KEYDOWN:
					printf("We got a keyboard event.\n");
					if(event.key.keysym.sym==I_OK){
						printf("Going back to Main Menu\n");
						state = MAIN_MENU;
						done = true;
					}
					break;
				default:
					printf("Unknown input, try again!\n");
					break;
			}
		}
	} 
}

void BustaEngine::characterSelect(){
	printf("entered character select screen\n");
	state = SP_GAME;
}

void BustaEngine::connectScreen(){
	printf("entered connection screen\n");
	graphics->drawConnection();
	state = CHAR_SELECT;
}

void BustaEngine::keyBindingMenu(){
	printf("entered key binding screen\n");
	state = OPTIONS;
}


void BustaEngine::soundsMenu(){
	printf("entered sounds menu\n");
	state = OPTIONS;
}


void BustaEngine::difficultyMenu(){
	printf("entered difficulty menu \n");
	state = OPTIONS;
}

void BustaEngine::playGame(){	
	printf("entered game loop");
	bool done = false;
	bool submit_control = true;
	int col_curr = 0;
	int row_curr = 0;
	int align_curr = 0;
	while(!done)
	{
		SDL_Event event;
		graphics->drawSPGame(gs);
		
		while ( SDL_PollEvent(&event) && !done)
		{
			graphics->drawSPGame(gs);
            col_curr = gs->getCursor()->getPosition()->getY();
	        row_curr = gs->getCursor()->getPosition()->getX();
			align_curr = gs->getCursor()->getAlignment();
			switch(event.type){
				case SDL_MOUSEMOTION:
					//printf("We got a motion event.\n");
					//printf("Current mouse position is: (%d, %d)\n", event.motion.x, event.motion.y);
					break;
				case SDL_KEYDOWN:
					
					if(event.key.keysym.sym==LEFT){
						moveLeft(col_curr, align_curr);
					}
					if(event.key.keysym.sym==RIGHT){
						moveRight(col_curr, align_curr);
					}
					if(event.key.keysym.sym==UP){
						moveUp(row_curr, align_curr);
					}
					if(event.key.keysym.sym==DOWN){
						moveDown(row_curr, align_curr);
					}
					if(event.key.keysym.sym==SWAP){
						swap(row_curr, col_curr, align_curr);
					}
					if(event.key.keysym.sym==ROTATE){
						rotate(row_curr, col_curr, align_curr);
					}
					if(event.key.keysym.sym==SWITCH_SUBMIT){
						submit_control++;
					}
					if(event.key.keysym.sym==SUBMIT){
						if(submit_control){
							submit_cus();
						}
						else submit_self();
					}
					if(event.key.keysym.sym==QUIT){
						printf("Exiting loop");
						done = true;
						state = EXIT;
					}
					break;
				case SDL_KEYUP:
					break;
				default:
					printf("Unhandled Event!\n");
					break;
			}
		}
	}  
}

void BustaEngine::displayRankings(){
	const int I_PLAY_AGAIN = SDLK_p;
	const int I_MAIN_MENU = SDLK_m;
	
	printf("entered ranking screen\n");
	graphics->drawRankings();

	bool done = false;

	while(!done)
	{
		SDL_Event event;

		while ( SDL_PollEvent(&event) && !done)
		{
			switch(event.type){
				case SDL_KEYDOWN:
					printf("We got a keyboard event.\n");
					if(event.key.keysym.sym==I_PLAY_AGAIN){
						printf("Play again, going back to Connection Screen\n");
						state = CONNECT;
						done = true;
					}
					else if(event.key.keysym.sym==I_MAIN_MENU){
						printf("Going back to Main Menu\n");
						state = MAIN_MENU;
						done = true;
					}
					break;
				default:
					printf("Unknown input, try again!\n");
					break;
			}
		}
	} 
}

char* BustaEngine::netInit(char* ip){
	char* error = network->init();

	if(error !=NULL){
		printf("Error initializing the networking client");
		return error;
	}

	char* conresp;

	printf("Connecting to: %s\n\n", ip);
	conresp = network->connect(ip, 65535);

	if(conresp!=NULL){
		printf("Error connecting to the specified server");
		return conresp;
	}



	return NULL;
}


int BustaEngine::moveLeft(int col_curr, int align_curr){	   
		if ((col_curr > 0)&& !(col_curr == 1 && align_curr == 0)){
			col_curr--;
			printf("movedLeft \n");
			gs->getCursor()->getPosition()->setY(col_curr);
			return 1;
		}
		return 0;
}

int BustaEngine::moveRight(int col_curr, int align_curr){
		if (((col_curr < WIDTH - 1) && !(col_curr == WIDTH - 2 && align_curr == 2))){
			col_curr++;
			printf("movedRight \n");
			gs->getCursor()->getPosition()->setY(col_curr);
			return 1;
		}
		return 0;
}

int BustaEngine::moveDown(int row_curr, int align_curr){
		if ((row_curr < HEIGHT - 1) && !(row_curr == HEIGHT - 2 && align_curr == 3)){
			row_curr++;
			printf("movedDown \n");
			gs->getCursor()->getPosition()->setX(row_curr);
			return 1;
		}
		return 0;
}
int BustaEngine::moveUp(int row_curr, int align_curr){
		if ((row_curr > 0) && !(row_curr == 1 && align_curr == 1)){
			row_curr--;
			printf("movedUp \n");
			gs->getCursor()->getPosition()->setX(row_curr);
			return 1;
		}
		return 0;
}

int BustaEngine::rotate(int row_curr, int col_curr, int align_curr){
	switch(align_curr){
		case 0: //left
			if (col_curr == WIDTH - 1 && row_curr == 0){
				align_curr = 3;
			}
			else if(row_curr == 0){
				align_curr = 2;	
			}
			else {
				align_curr = (align_curr + 1)%4;
			}
			break;
		case 2: //right
			if (col_curr == 0 && row_curr == HEIGHT - 1){
				align_curr = 1;
			}
			else if(row_curr == HEIGHT - 1){
				align_curr = 0;
			}
			else {
				align_curr = (align_curr + 1)%4;
			}
			break;
		case 1: // up
			if (col_curr == WIDTH - 1 && row_curr == HEIGHT - 1){
				align_curr = 0;
			}
			else if(col_curr == WIDTH - 1){
				align_curr = 3;
			}
			else{ 
				align_curr = (align_curr + 1)%4;
			}
			break;
		case 3: // down
			if (col_curr == 0 && row_curr == 0){
				align_curr = 2;
			}
			else if(col_curr == 0){
				align_curr = 1;
			}
			else{ 
				align_curr = (align_curr + 1)%4;
			}
			break;
	}
	gs->getCursor()->setAlignment(align_curr);
	return 1;
}

int BustaEngine::swap(int row_curr, int col_curr, int align_curr){ 
	BustaModelFoodPiece* curr_cursor = gs->getBoard(row_curr, col_curr);
	BustaModelFoodPiece* temp = curr_cursor;
	//printf("align value: %d\n", align_curr);
	switch (align_curr){
		case 0: //Left
			 if (col_curr > 0){
				BustaModelFoodPiece* second_cursor = gs->getBoard(row_curr, col_curr - 1);
				gs->setBoard(row_curr, col_curr, second_cursor);
				gs->setBoard(row_curr, col_curr - 1, temp);
				return 1;
			 }
			 break;
		case 2: //Right
			 if (col_curr < 4){
				BustaModelFoodPiece* second_cursor = gs->getBoard(row_curr, col_curr + 1);
				gs->setBoard(row_curr, col_curr, second_cursor);
				gs->setBoard(row_curr, col_curr + 1, temp);			
				return 1;
			 }
			 break;
		case 1: //Up
			 if (row_curr > 0){
			    BustaModelFoodPiece* second_cursor = gs->getBoard(row_curr - 1, col_curr);
				gs->setBoard(row_curr, col_curr, second_cursor);
				gs->setBoard(row_curr - 1, col_curr, temp);			
				return 1;
			 }
			 break;
		case 3: //Down
			 if (row_curr < 4){
			    BustaModelFoodPiece* second_cursor = gs->getBoard(row_curr + 1, col_curr);
				gs->setBoard(row_curr, col_curr, second_cursor);
				gs->setBoard(row_curr + 1, col_curr, temp);			
				return 1;
			 }
			 break;
	}
	
	return 0;
}

int BustaEngine::submit_cus(){
	BustaModelCompleteSandwich* yum = new BustaModelCompleteSandwich();
	for (int i = 0; i < WIDTH; i++){
		BustaModelFoodPiece* ingr = gs->getBoard(HEIGHT - 1, i);
		yum->setIngredient(ingr, i);
	}

	bool cust_sand = false;
	int curr_queue = NUMCUSTOMERS - 1;
	while (curr_queue >= 0){
		BustaModelFoodPiece** order = gs->getCustomer(curr_queue)->getOrder();
		for (int i = 0; i < WIDTH ; i++ ){
			FoodType a = yum->getIngredient(i)->getType();
			FoodType b = order[i]->getType();
			if (a == b){ cust_sand = true; }
		}
		curr_queue--;
		if (!cust_sand){ break; }
	}
		if (cust_sand){
			BustaModelCustomer* c = gs->getCustomer(curr_queue);
			gs->removeCustomer(c);
			int score_multiplier = c->getTime();
			int add_score = 10 * score_multiplier;
			gs->setScore(add_score);
			gs->getTipJar()->addTips(2);

		}

		else{
			int curr_health = gs->getChef()->getHealth();
			gs->getChef()->setHealth(curr_health - 4);
		}
	
	submit_board();
////////////////////////////////////////////////////////////////
	BustaModelCustomer* a_cust = new BustaModelCustomer();
	gs->addCustomer(a_cust);
///////////////////////////////////////////////////////////////
	return 0;
}

	int BustaEngine::submit_self(){
		BustaModelCompleteSandwich* yum = new BustaModelCompleteSandwich();
		for (int i = 0; i < WIDTH; i++){
			BustaModelFoodPiece* ingr = gs->getBoard(HEIGHT - 1, i);
			yum->setIngredient(ingr, i);
		}
		submit_board();
		return 0;
	}

void BustaEngine::submit_board(){
	for (int row = HEIGHT - 2; row >= 0; row--){
		for(int col = 0; col < WIDTH; col++){
			BustaModelFoodPiece* temp = gs->getBoard(row, col);
			gs->setBoard(row +1 , col, temp);
		}
	}
	for (int col = 0; col < WIDTH; col++){
		int r = rand();
		int ingred = r % NUMFOODPIECES ;
		BustaModelFoodPiece* new_top = new BustaModelFoodPiece((FoodType)ingred, 0);
		gs->setBoard(0 , col, new_top);
	}
}

bool BustaEngine::buttonSelected(Button b, int x, int y){
	if(x>=b.box.x&&x<=b.box.x+b.box.w){
		if(y>=b.box.y&&y<=b.box.y+b.box.h){
			return true;
		}
	}
	return false;
}

BustaModelGameState* BustaEngine::getGameState(){
	return gs;
}