/* 
 * BustaGraphics.h
 * Busta' Sandwich Graphics Engine
 * Authors: Julian Chan, Thomas Patecky
 *
 * Created: July 13, 2005
 * Last Modified July 15, 2005
 */

#ifndef BUSTAGRAPHICS_H
#define BUSTAGRAPHICS_H


#include "../../SDL/include/SDL.h"
#include "../models/BustaModelGameState.h"
#include "BustaGraphicsSprites.h"
#include "BustaGraphicsLayout.h"
#include "Font.h"

struct Button {
	SDL_Rect box;
	Button* nextLink;
	Button* prevLink;
};

/**
 * @class BustaGraphics
 * @brief contains methods for displaying graphics
 *
 */
class BustaGraphics
{
	private:
		//The screen to draw things on
		SDL_Surface* myScreen;
		//All the sprites
		BustaGraphicsSprites* mySprites;
		//BMP font
		SFont* myFont;

		//How many frames have been rendered in total
		int overall_anim_frame;

		int chef_anim_frame;
		int customer_anim_frame;

		//Private drawing functions

		/**
		 * @brief Draws board
		 *
		 *
		 * @param gs A pointer to the BustaModelGameState.
		 */
		void drawBoard(BustaModelGameState* gs);

		/**
		 * @brief Draws a food tile
		 *
		 *
		 * @param food_piece A pointer to the BustaModelFoodPiece.
		 * @param row Row to place tile in.
		 * @param col Column to place tile in.
		 */
		void drawFoodPiece(BustaModelFoodPiece* food_piece, int x, int y);

		/**
		* @brief Draws the tip jar
		* @param tj A pointer to the Tip Jar Model.
		*/
		void drawTipJar(int x, int y, BustaModelTipJar* tj);

		/**
		* @brief Draws the score
		* @param score The Score.
		*/
		void drawScore(int score);
		
		/**
		* @brief Draws the cursor 
		* @param row y-pos.
		* @param col x-pos.
		* @param swap_alignment alignment of the swap cursor {LEFT, RIGHT, UP, DOWN}
		*/
		void drawCursor(int row, int col, int swap_alignment);
	
		/**
		* @brief clears the screen with the specified color
		* @param 
		*/
		void clearScreen(Uint32 color);

		/**
		* @brief Draws the chef
		* @param c A pointer to the Chef Model.
		*/

		void drawChef(BustaModelChef* c);	
		
		/**
		* @brief Draws the chef
		* @param x x-pos of the top-left corner of the bar
		* @param y y-pos
		* @param health current health of the chef.
		* @param max_health the max possible health of the chef
		*/
		void drawHealthBar(int x, int y, int health, int max_health);
		
		/**
		* @brief Draws the customer
		* @param x x-pos of the top-left corner of the first piece
		* @param y y-pos
		* @param customer a pointer to the current customer
		*/
		void drawCustomer(int x, int y, BustaModelCustomer* customer);

		/**
		* @brief Draws the customer's order with overlapping food pieces
		* @param x x-pos of the top-left corner of the first piece
		* @param y y-pos
		* @param order_array customer's order
		*/
		void drawOrder(int x, int y, BustaModelFoodPiece** order_array);
		
		/**
		 * @brief update a rectangle on the screen
		 * @param rect a rectangle on the screen
		 * @param rectSurface image to update the rectangle with
		 */
		void updateRect(SDL_Rect* rect, SDL_Surface* rectSurface);

				
		/**
		 * @brief intialize the button parameters
		 */
		void initButtonLocs();
	
	public:
		BustaGraphics();
		BustaGraphics(int a);
		BustaGraphics(SDL_Surface* screen);

		/**
		* @brief Draws a window containg a singleplayer game
		* @param gs A pointer to the Game State.
		*/
		void drawSPGame(BustaModelGameState* gs);

		/**
		* @brief Draws a window containg a multiplayer game
		* @param gs A pointer to the Game State.
		*/
		void drawMPGame(BustaModelGameState gs);
		/**
		* @brief Draws the opponent's game board
		* @param gs A pointer to the Game State.
		*/
		void drawOpponentBoard(BustaModelGameState gs);

		//Other screens

		/**
		* @brief Draws the main screen
		*/
		void drawMainScreen(MainScrnButton selected);

		/**
		* @brief Draws the chef selection screen
		*/
		void drawChefSelelection();

		/**
		* @brief Draws the high scores screen
		*/
		void drawHighScores();

		/**
		* @brief Draws the rankings screen
		*/
		void drawRankings();

		/**
		* @brief Draws the options screen
		*/
		void drawOptions();

		/**
		* @brief Draws the connect screen
		*/
		void drawConnection();

		/**
		* @brief Data Structure containing the button locations and links
		*/
		Button bMainScrn[MS_NUM_BUTTONS];

};


#endif
