/*	Filename:		BustaModelCursor.h
 *	Description:	Busta' Sandwich Cursor model.
 *
 */

#ifndef BUSTA_SANDWICH_CURSOR
#define BUSTA_SANDWICH_CURSOR

#include "BustaModelPosition.h"


/**
 * @enum CursorAlignment
 * @brief Alignment of cursor
 *
 *	Alignment of 2nd cursor location in relation to first in clockwise order (left, up, right, down)
 */
enum CursorAlignment{LEFT,UP,RIGHT,DOWN}; 

/**
 * @class BustaModelCursor
 * @brief Game Cursor
 *
 * Cursor for gameboard, has position and alignment, which specifies secondary position.
 * Modifier includes whether chef is confused, etc.
 */
class BustaModelCursor
{
	public:
		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaModelCursor();


		/**
		 * @brief Destructor
		 *
		 *	Deletes position variable
		 */
		~BustaModelCursor();

		/**
		 * @brief Gets cursor position
		 *
		 * Gets the current board position of the cursor
		 */
		BustaModelPosition* getPosition();

	
		/**
		 * @brief Sets Positino of cursor
		 * sets Position of cursor via x,y coordinates
		 */
		void setPosition(int x, int y);


		/**
		 * @brief gets alignment of cursor
		 * Gets the alignment of the cursor, returning enum type of alignment
		 */
		int getAlignment();

		/**
		 * @brief sets alignment of cursor
		 * sets the alignment of the cursor 
		 */
		void setAlignment(int a);
		
		/**
		 * @brief gets cursor modifier
		 * Gets current modifier on cursor - this affects whether moves are made via regular keybindings or not
		 */
		int getModifier();

		/**
		 * @brief sets cursor modifier
		 * Sets the cursor modifier
		 */
		void setModifier(int m);

	private:
		BustaModelPosition* pos;
		int alignment;
		int modifier;
};

#endif