/*	Filename:		BustaNetClient.cpp
 *	Description:	Busta' Sandwich client module implementation
 *
 */

#include "BustaNetClient.h"
#include <stdlib.h>

BustaNetClient::BustaNetClient()
{
	// no server
	serverName = NULL;

	/* empty socket connections */
	tcpsock = NULL;
	socketset = NULL;

	/* Initialize message queues */
	SandwichQ = new BustaNetMsgQ();
	PriorityQ = new BustaNetMsgQ();

	/* Create a multiplayer game state */
	MPGameState = new BustaModelMPGameState();
}

BustaNetClient::~BustaNetClient()
{
	// delete message queues
	delete(SandwichQ);
	delete(PriorityQ);
	delete(MPGameState);

	// Shutdown SDLnet
	SDLNet_Quit();
}

char* BustaNetClient::init()
{
	// Initialize SDLnet
	if	(SDLNet_Init() == -1) 
		return SDLNet_GetError();	// On error, return the error message

	return NULL;	// return null on no errors
}

char* BustaNetClient::connect(char* IP, Uint16 serverPort)
{
	// connection status
	char* connected;

	// Resolve the server IP to be stored
	SDLNet_ResolveHost(&serverIP, IP, serverPort);

	// If the address cannot be resolved
	if (serverIP.host == INADDR_NONE) 
	{
		// unable to resolve server address
		connected = "Unable to resolve server address";
	} 
	else
	{
		// Attempt a connection to the server
		tcpsock = SDLNet_TCP_Open(&serverIP);

		// if a connection cannot be made with the server
		if (tcpsock == NULL) 
		{
			// unable to connect to server
			connected = "Unable to connect to server";
		} 
		else 
		{
			// successfully connected
			connected = NULL;
		}
	}

	// return connection status
	return connected;
}

void BustaNetClient::disconnect()
{
	// send disconnect message
	char type = 'd';						// type
	SDLNet_TCP_Send(tcpsock, &type, 1);		// send disconnect type msg

	// Close socket connection
	SDLNet_TCP_Close(tcpsock);
	tcpsock = NULL;
}

BustaModelMPGameState* BustaNetClient::getMPGS()
{
	return MPGameState;
}

BustaNetMsgQ* BustaNetClient::getPriorityQ()
{
	return PriorityQ;
}

BustaNetMsgQ* BustaNetClient::getSandwichQ()
{
	return SandwichQ;
}

char* BustaNetClient::getServerName()
{
	return serverName;
}

MPConnectResponse* BustaNetClient::joinGame(char* playerName)
{
	MPConnectResponse* mpcs;	// connection response from server

	char type ='a';			// join game network message type
	int nameLen;			// length of player's name
	cs_join_attempt ja;		// join network message

	/* enclose prefered player name into network message */
	nameLen = strlen((const char*)playerName);	// find the length of the player's name
	if (nameLen > 20)							// if the name is longer than 20 characters
	{
		nameLen = 20;								// truncate the length

		for (int i = 0; i < nameLen; i++)			// Copy player name to enclosing message
			ja.clientName[i] = playerName[i];
		ja.clientName[20] = '\0';					// Add delimiting null
	}
	else
	{
		strcpy(ja.clientName, (const char*)playerName);
	}

	/* send the message */
	SDLNet_TCP_Send(tcpsock, &type, 1);					// send the join game type
	SDLNet_TCP_Send(tcpsock, &ja, 21*(sizeof(char)));	// send the msg

	/* Server is to send back joingame response */
	// query socket set for 500ms
	if (SDLNet_CheckSockets(socketset, 500) == 0)		// if no activity in 500ms
	{
		// no response from server
		mpcs = new MPConnectResponse(false, "Timeout", 0);
	}
	else
	{
		

	}


	return mpcs;
}

void BustaNetClient::processNet()
{


}

void BustaNetClient::sendCharSelect(BustaModelCharacter character)
{
	char type = 'b';		// the type of the message to send
	cs_character_select cs;	// the character select msg to send to the server

	cs.character = character;	// enclose the selection in the msg

	SDLNet_TCP_Send(tcpsock, &type, 1);							// Send the type of the msg
	SDLNet_TCP_Send(tcpsock, &cs, sizeof(cs_character_select));	// send the selection	
}

void BustaNetClient::sendCursor(BustaModelPosition* cursor)
{
	char type = 'c';	// the type of the message to send
	cs_cursor cur;		// the cursor to send

	// enclose the cursor location in the net message
	cur.cursorLoc.x1 = cursor->getX();
	cur.cursorLoc.y1 = cursor->getY();

	SDLNet_TCP_Send(tcpsock, &type, 1);					// send the type of the message
	SDLNet_TCP_Send(tcpsock, &cur, sizeof(cs_cursor));	// send the cursor
}

void BustaNetClient::sendGameOver()
{
	char type = 'e';						// game over msg type
	SDLNet_TCP_Send(tcpsock, &type, 1);		// send the type of the message
}

void BustaNetClient::sendOpponentViewReq(int opponentID)
{
	char type = 'f';				// type of message to send
	cs_opponent_view_select ovs;	// message to send
	
	// enclose selection in message
	ovs.opponentID = opponentID;

	SDLNet_TCP_Send(tcpsock, &type, 1);					// send the type of the message
	SDLNet_TCP_Send(tcpsock, &ovs, sizeof(cs_opponent_view_select));	// send the msg
}

void BustaNetClient::sendReady()
{
	char type = 'g';						// ready msg type
	SDLNet_TCP_Send(tcpsock, &type, 1);		// send the type of the message
}

void BustaNetClient::sendSandwich(int opponentID, SpecialSandwich sandwich)
{
	char type = 'h';		// type of the message to send
	cs_sandwich_send ss;	// the message to send

	// enclose the the sandwich in the message
	ss.targetPlayer = opponentID;
	ss.sandwich = sandwich;

	SDLNet_TCP_Send(tcpsock, &type, 1);							// send the type of the message
	SDLNet_TCP_Send(tcpsock, &ss, sizeof(cs_sandwich_send));	// send the msg
}

void BustaNetClient::sendState(BustaModelGameState* gs)
{
	int i, j;
	char type = 'a';
	netGameState netGS;

	//score
	netGS.score = gs->getScore(); 

	//game board
	for(i = 0; i < HEIGHT; i++)
	{
		for(j = 0; j < WIDTH; j++)
		{
			netGS.board[i][j].state = (gs->getBoard(i, j))->getState();
			netGS.board[i][j].type = (gs->getBoard(i, j))->getType();
		}
	}

	//customer queue
	for(i = 0; i < NUMCUSTOMERS; i++)
	{
		for(j = 0; j < WIDTH; j++)
		{
			netGS.customers[i].order[j].state = ((gs->getCustomer(i))->getOrder())[j]->getState();
			netGS.customers[i].order[j].type = ((gs->getCustomer(i))->getOrder())[j]->getType();
		}
		netGS.customers[i].timeleft = (gs->getCustomer(i))->getTime();
	}

	//tip jar
	netGS.tips.tips = (gs->getTipJar())->getTips();
	netGS.tips.fullamount = (gs->getTipJar())->getSize();

	//chef
	netGS.chef.health = (gs->getChef())->getHealth();

	//cursor
	netGS.cursor.alignment	= (gs->getCursor())->getAlignment();
	netGS.cursor.modifier	= (gs->getCursor())->getModifier();
	netGS.cursor.pos.x1		= ((gs->getCursor())->getPosition())->getX();
	netGS.cursor.pos.y1		= ((gs->getCursor())->getPosition())->getY();

	SDLNet_TCP_Send(tcpsock, &type, 1);
	SDLNet_TCP_Send(tcpsock, &netGS, sizeof(netGameState));
	//FYI: sizeof(netGameState) =  364 bytes as of 7/22/05
}
