/*	Filename:		BustaNetServer.cpp
 *	Description:	Busta' Sandwich server module implementation
 *
 */

#include <stdlib.h>
#include "BustaNetServer.h"

// ------------------------------- Public Functions ------------------------------------------
BustaNetServer::BustaNetServer()
{
	/* Initialize variables */
	clients = NULL;			// No clients
	clientSocketSet = NULL;	// No socket set for clients
	maxClients = 0;			// No clients allowed
	numClients = 0;			// No clients currently connected
	serverName = NULL;		// No server name
	serverSocketSet = NULL;	// No socket set for server messages
	servsock = NULL;		// No socket for server messages

	debug = false;		// DEBUG - debug output modifier
}

BustaNetServer::~BustaNetServer()
{
	// Close tcp server socket
	SDLNet_TCP_Close(servsock);

	// Close tcp client sockets, and delete client list, full and brief GS
	for (int i = 0; i < maxClients; i++) //maxClients should be deleted since they are created
	{
		SDLNet_TCP_Close(clients[i]->socket);		// Close client socket
		delete clients[i];							// Delete client entry
	}

	//Delete gamestates
	delete[] fullGameStates;
	delete[] briefGameStates;
	//might need to delete client pointers as well here

	// Shutdown SDL
	SDL_Quit();
}

char* BustaNetServer::init(Uint16 servport)
{
	client* oneClient;				// client record pointer

	// Create client list
	clients	= (client**)(malloc(maxClients*(sizeof(*oneClient))));
	for (int i = 0; i < maxClients; i++)
	{		
		// Create new client record
		oneClient = (struct client*)(malloc(sizeof(*oneClient)));

		// Set Client defaults
		oneClient->active = false;
		oneClient->id_number = 0;
		oneClient->name = NULL;
		oneClient->ready = false;
		oneClient->socket = NULL;

		// Add client record to client list
		clients[i] = oneClient;
	}

	// Initialize fullGameStates and briefGameStates
	fullGameStates =  new netGameState*[maxClients];
	briefGameStates = new netOpponent[maxClients];
	for(int i = 0; i < maxClients; i++)
	{
		
		fullGameStates[i] = new netGameState;
		fullGameStates[i]->score = 0;

		briefGameStates[i].character = 0;
		briefGameStates[i].health = 0;
	}
	
	// Initialize SDL
	if (SDL_Init(0) < 0)
		return SDL_GetError();		// Return error msg on SDL error

	// Initialize SDLnet
	if (SDLNet_Init() == -1) 
		return SDLNet_GetError();	// Return error msg on SDLNet error

	// Initialize SDL timer subsystem
	if (SDL_Init(SDL_INIT_TIMER) < 0)
		return SDL_GetError();		// Return error msg on SDL timer error

	// Allocate the socket set
	clientSocketSet = SDLNet_AllocSocketSet(1);		// client socket set
	serverSocketSet = SDLNet_AllocSocketSet(1);		// server socket set

	// If we cannot create the server or client socket sets, return the error msg
	if ((clientSocketSet == NULL) || (serverSocketSet == NULL)) 
		return SDLNet_GetError();
		
	// Create the server socket
	SDLNet_ResolveHost(&serverIP, NULL, servport);			// resolve the server's ip with the specified port

	// Open the server socket
	servsock = SDLNet_TCP_Open(&serverIP);

	// If we cannot create the server socket, return the error msg
	if (servsock == NULL) 
		return SDLNet_GetError();

	// Add the server socket to the server socket set
	SDLNet_TCP_AddSocket(serverSocketSet, servsock);

	// return null on no error
	return NULL;
}

//this function will have to be changed, but at this point it tests the connection accept packets
void BustaNetServer::sendAccept(int to_client, bool response, char* message)
{
	if(response)
	{
		char type = 'I';
		sc_new_connection msg1;

		msg1.playerID = clients[to_client]->id_number;
		strcpy(msg1.playerName, clients[to_client]->name);

		SDLNet_TCP_Send(clients[to_client]->socket, &type, 1);
		SDLNet_TCP_Send(clients[to_client]->socket, &msg1, sizeof(sc_new_connection));
		

		type = 'J';
		sc_server_settings msg2;

		strcpy(msg2.serverName, getName());
		
		SDLNet_TCP_Send(clients[to_client]->socket, &type, 1);
		SDLNet_TCP_Send(clients[to_client]->socket, &msg2, sizeof(sc_server_settings));
	}
	else
	{
		char type = 'I';
		sc_new_connection msg1;

		msg1.playerID = -1;
		strcpy(msg1.playerName, message);

		SDLNet_TCP_Send(clients[to_client]->socket, &type, 1);
		SDLNet_TCP_Send(clients[to_client]->socket, &msg1, sizeof(sc_new_connection));
	}
}

void BustaNetServer::sendCharacterSelectedAllO(client* sel_client, BustaModelCharacter character)
{
	for(int i = 0; i < numClients; i++)
	{
		if(clients[i]->id_number != sel_client->id_number)
			sendCharacterSelected(clients[i], sel_client, character);
	}
}

void BustaNetServer::sendCharacterSelectCursorAllO(client* sel_client, BustaModelPosition* cursor)
{
	for(int i = 0; i < numClients; i++)
	{
		if(clients[i]->id_number != sel_client->id_number)
			sendCharacterSelectCursor(clients[i], sel_client, cursor);
	}
}

void BustaNetServer::sendCharacterSelectModeAll()
{
	for(int i = 0; i < numClients; i++)
	{
		sendCharacterSelectMode(clients[i]);
	}
}

void BustaNetServer::sendDisconnectedAllO(client* disc_client, char* message)
{
	for(int i = 0; i < numClients; i++)
	{
		if(clients[i]->id_number != disc_client->id_number)
			sendDisconnected(clients[i], disc_client, message);
	}
}

void BustaNetServer::sendGameStartAll()
{
	for(int i = 0; i < numClients; i++)
	{
		sendGameStart(clients[i]);
	}
}

void BustaNetServer::sendPlayerLostAllO(client* lost_player)
{
	for(int i = 0; i < numClients; i++)
	{
		if(clients[i]->id_number != lost_player->id_number)
			sendPlayerLost(clients[i], lost_player);
	}
}

void BustaNetServer::sendSandwichAllO(client* target_client, client* from_client, SpecialSandwich sandwich)
{
	for(int i = 0; i < numClients; i++)
	{
		if(clients[i]->id_number != from_client->id_number)
			sendSandwich(clients[i], target_client, from_client, sandwich);
	}
}

void BustaNetServer::sendUpdateAll()
{
	for(int i = 0; i < numClients; i++)
	{
		sendUpdate(clients[i]);
	}
}

void BustaNetServer::setDebug(bool OnOff)
{
	debug = OnOff;
}

void BustaNetServer::setMaxClients(int maxC)
{
	maxClients = maxC;
}

int BustaNetServer::getMaxClients()
{
	return maxClients;
}

void BustaNetServer::setName(char* servname)
{
	serverName = servname;
}

char* BustaNetServer::getName()
{
	return serverName;
}

int BustaNetServer::setStateAccept()
{
	TCPsocket newsock;		// Socket to assign to new clients

	// Begin Accepting Connections
	for (;;)
	{
		/* If there are at least two clients who are ready, exit accept state */
		if (numClients >= 2)
		{
			int clientsRdy = 0;							// number of ready clients

			for (int i = 0; i < maxClients; i++)		// Count number of ready clients
			{
				if (clients[i]->ready)
					clientsRdy++;
			}

			if (clientsRdy == numClients)				// If all clients ready, exit accept state
				return 1;
		}

		// Check for input on server socket
		SDLNet_CheckSockets(serverSocketSet, 0);

		// If input on socket, we have a connection
		if (SDLNet_SocketReady(servsock))
		{
			// Keep the socket used to connect to the server
			newsock = SDLNet_TCP_Accept(servsock);

			// DEBUG - Notify console of connection attempt
			if (debug)
				printf("Incoming Connection\n");

/**/		addClient(*SDLNet_TCP_GetPeerAddress(newsock),newsock, numClients, "a");
		}

		// Wait to poll socket again (80 ms)
		SDL_Delay(80);
	}

	// Error return
	return 0;
}

int BustaNetServer::setStateCharSelect()
{
	//for(;;)
	//{
	//	// Check for input on server socket
	//	SDLNet_CheckSockets(serverSocketSet, 0);

	//	// If there is activity, check all the sockets
	//	if (SDLNet_SocketReady(servsock))
	//	{
	//		for(int i = 0; i < numClients; i++)
	//		{
	//			//SDLNet_TCP_Recv(clients[i]->socket);
	//		}
	//	}

	//	// Wait to poll socket again (80 ms)
	//	SDL_Delay(80);
	//}

	return 0;
}

int BustaNetServer::setStateGame()
{
	return 0;
}

// ------------------------------- Private Functions ------------------------------------------

void BustaNetServer::addClient(IPaddress IP, TCPsocket socket, int id, char* clientName)
{
	int clientRecord;		// Open client record index in client list

	// Locate an empty client record
	for (clientRecord = 0; clientRecord < maxClients; clientRecord++)
	{
		if (!(clients[clientRecord]->active))
			break;								// break out once an open slot is found
	}

	// Create a record of the client
	clients[clientRecord]->active = true;
	clients[clientRecord]->address = IP;
	clients[clientRecord]->char_selected = false;
/**/clients[clientRecord]->focus_opp = fullGameStates[id]; //default focus opponent is self for now
	clients[clientRecord]->id_number = id;
	clients[clientRecord]->name = clientName;
/**/clients[clientRecord]->ready = true; //false
	clients[clientRecord]->socket = socket;

	// Add the client socket to the socket set
	SDLNet_TCP_AddSocket(clientSocketSet, clients[clientRecord]->socket);

	// Increment number of clients connected
	numClients++;

	// DEBUG - output connection info for client
	if (debug)
	{
		printf("Client Connected\n");
		printf("\tAddress: %x\n", clients[clientRecord]->address.host);
		printf("\tID Number: %i\n", clients[clientRecord]->id_number);
		printf("\tName: %s\n", clients[clientRecord]->name);
		printf("\n");
	}
}

void BustaNetServer::sendCharacterSelected(client* to_client, client* sel_client, BustaModelCharacter character)
{
	char type = 'F';
	sc_character_selected msg;

	msg.selectingPlayer = sel_client->id_number;
	msg.character = character;

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, &msg, sizeof(sc_character_selected));
}

void BustaNetServer::sendCharacterSelectMode(client* to_client)
{
	char type = 'D';

	SDLNet_TCP_Send(to_client->socket, &type, 1);
}

void BustaNetServer::sendCharacterSelectCursor(client* to_client, client* sel_client, BustaModelPosition* cursor)
{
	char type = 'G';
	sc_character_select_cursor msg;

	msg.playerID= sel_client->id_number;
	msg.cursorLoc.x1 = cursor->getX();
	msg.cursorLoc.y1 = cursor->getY();

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, &msg, sizeof(sc_character_select_cursor));
}

void BustaNetServer::sendDisconnected(client* to_client, client* disc_client, char* message)
{
	char type = 'C';
	sc_player_disconnect msg;

	msg.playerID = disc_client->id_number;
	strcpy(msg.discMessage, message);

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, &msg, sizeof(sc_player_disconnect));
}

void BustaNetServer::sendGameStart(client* to_client)
{
	char type = 'B';

	SDLNet_TCP_Send(to_client->socket, &type, 1);
}

void BustaNetServer::sendPlayerLost(client* to_client, client* lost_player)
{
	char type = 'E';
	sc_player_lost msg;

	msg.playerID = lost_player->id_number;

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, &msg, sizeof(sc_player_lost));
}

void BustaNetServer::sendSandwich(client* to_client, client* target_client, client* from_client, SpecialSandwich sandwich)
{
	char type = 'H';
	sc_sandwich_send msg;

	msg.originatingPlayer = from_client->id_number;
	msg.targetPlayer = target_client->id_number;
	msg.sandwich = sandwich;

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, &msg, sizeof(sc_sandwich_send));
}

void BustaNetServer::sendUpdate(client* to_client)
{
	char type = 'A';

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, to_client->focus_opp, sizeof(netGameState));
	SDLNet_TCP_Send(to_client->socket, briefGameStates, maxClients*sizeof(netOpponent));
}